using System;
using System.Drawing;
using System.Windows.Forms;

// Delegates
public delegate void ClearHandler();
public delegate void ModeChangedHandler(Modes m);
public delegate void DisplayOptionsChangedHandler(bool showControlPoints, bool showCurvePoints, bool showConstructionPoints);

class ControlPanel : Panel
{
	// Members
	private Button clear;
	private ComboBox mode;
	private Label x, y;
	private CheckBox showControlPoints, showCurvePoints, showConstructionPoints;
	
    // Events
    public event ClearHandler Clear;
    public event ModeChangedHandler ModeChanged;
    public event DisplayOptionsChangedHandler DisplayOptionsChanged;
	
	public ControlPanel()
	{
		int margin = 5;
		// this.BackColor = Color.SlateGray;

		// Add controls
		this.clear = new Button();
		this.clear.Text = "Clear";
		this.clear.Location = new Point(margin, margin);
		this.clear.Click += new EventHandler(this.publishClear);
		this.Controls.Add(this.clear);

		this.mode = new ComboBox();
		this.mode.Location = new Point(this.clear.Right + margin, margin);
		this.mode.DropDownStyle = ComboBoxStyle.DropDownList;
		this.mode.Items.Add("Create");
		this.mode.Items.Add("Edit");
		this.mode.Items.Add("Delete");
		this.mode.SelectedIndex = 0;
		this.mode.SelectedValueChanged += new EventHandler(this.publishMode);
		this.Controls.Add(this.mode);
		
		this.x = new Label();
		this.x.BorderStyle = BorderStyle.Fixed3D;
		this.x.Width = 50;
		this.x.Location = new Point(this.mode.Right + margin, margin);
		this.x.TextAlign = ContentAlignment.MiddleCenter;
		this.Controls.Add(this.x);
		
		this.y = new Label();
		this.y.BorderStyle = BorderStyle.Fixed3D;
        this.y.Width = 50;
		this.y.TextAlign = ContentAlignment.MiddleCenter;
		this.y.Location = new Point(this.x.Right + margin, margin);
		this.Controls.Add(this.y);
		
		this.showControlPoints = new CheckBox();
		this.showControlPoints.Width = 130;
		this.showControlPoints.Text = "Show Control Points";
		this.showControlPoints.Checked = true;
		this.showControlPoints.Location = new Point(this.y.Right + margin, margin);
		this.showControlPoints.CheckedChanged += new EventHandler(this.publishDisplayOptions);
		this.Controls.Add(this.showControlPoints);

		this.showCurvePoints = new CheckBox();
		this.showCurvePoints.Width = 120;
		this.showCurvePoints.Text = "Show Curve Points";
        this.showCurvePoints.Checked = true;
		this.showCurvePoints.Location = new Point(this.showControlPoints.Right + margin, margin);
        this.showCurvePoints.CheckedChanged += new EventHandler(this.publishDisplayOptions);
		this.Controls.Add(this.showCurvePoints);

		this.showConstructionPoints = new CheckBox();
		this.showConstructionPoints.Width = 160;
		this.showConstructionPoints.Text = "Show Construction Points";
		this.showConstructionPoints.Location = new Point(this.showCurvePoints.Right + margin, margin);
        this.showConstructionPoints.CheckedChanged += new EventHandler(this.publishDisplayOptions);
		this.Controls.Add(this.showConstructionPoints);
		
		this.Height = margin * 2 + this.clear.Height;
	}
	
	private void publishClear(object o, EventArgs e)
	{
	    if (this.Clear != null)
	        this.Clear();
	}

    private void publishMode(object o, EventArgs e)
	{
	    Modes m = (Modes) Enum.Parse(typeof(Modes), this.mode.Text);
	    
	    if (this.ModeChanged != null)
	        this.ModeChanged(m);
	}

    private void publishDisplayOptions(object o, EventArgs e)
	{
	    if (this.DisplayOptionsChanged != null)
	        this.DisplayOptionsChanged(this.showControlPoints.Checked, this.showCurvePoints.Checked, this.showConstructionPoints.Checked);
	}
	
	public void SetXY(object o, MouseEventArgs e)
	{
	    this.x.Text = e.X.ToString();
        this.y.Text = e.Y.ToString();
	}
}