using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

class ViewPanel : Panel
{
    private BezierModel b;
    private bool showControlPoints;
    private bool showCurvePoints;
    private bool showConstructionPoints;
    private int node2Drag;
    
    public ViewPanel(BezierModel b)
    {
        this.b = b;
        this.BackColor = Color.AliceBlue;
        //this.SetStyle(ControlStyles.DoubleBuffer, true);
        this.DoubleBuffered = true;
        this.showControlPoints = true;
        this.showCurvePoints = true;
        this.showConstructionPoints = false;
        this.node2Drag = -1;
    }
    
    public void ChangeDiplayOptions(bool showControlPoints, bool showCurvePoints, bool showConstructionPoints)
    {
        this.showControlPoints = showControlPoints;
        this.showCurvePoints = showCurvePoints;
        this.showConstructionPoints = showConstructionPoints;
        this.Invalidate();
    }
    
    // Move nodes
	protected override void OnMouseDown(MouseEventArgs e)
	{
		base.OnMouseDown(e);

		// Create nodes
		if (this.b.Mode == Modes.Create)
			this.b.AddPoint(new Point(e.X, e.Y));
            
		// Edit / Delete nodes
		if (this.b.Mode > Modes.Create)
		{
			if (this.b.ControlPoints.Count == 0)
				return;
            
			// Get the control point that has been clicked
			int i = -1;
			while (++i < this.b.ControlPoints.Count)
			{
				Point p = (Point)this.b.ControlPoints[i];
				if ((e.X - (p.X - 3)) < 5 &&
					(e.X - (p.X - 3)) > 0 &&
					(e.Y - (p.Y - 3)) < 5 &&
					(e.Y - (p.Y - 3)) > 0)
				{
					if (this.b.Mode == Modes.Edit)
						this.node2Drag = i;
					else if(this.b.Mode == Modes.Delete)
						this.b.RemovePoint((Point)this.b.ControlPoints[i]);

					Console.WriteLine("OnMouseDown: Got node {0} at X: {1} / Y: {2}", i, e.X, e.Y);
					break;
				}
			}
		}
	}
	
	protected override void OnMouseMove(MouseEventArgs e)
	{
		base.OnMouseMove(e);

		// Move nodes
		if (this.node2Drag == -1)
			return;
		
		this.b.MovePoint(node2Drag, new Point(e.X, e.Y));
		Console.WriteLine("OnMouseMove: Move node {0} at X: {1} / Y: {2}", node2Drag, e.X, e.Y);
	}

	protected override void OnMouseUp(MouseEventArgs e)
	{
		base.OnMouseUp(e);
		
		// Move nodes
		if (this.node2Drag == -1)
			return;

		this.b.MovePoint(node2Drag, new Point(e.X, e.Y));
		Console.WriteLine("OnMouseUp: Placed node {0} at X: {1} / Y: {2}", node2Drag, e.X, e.Y);
		
		this.node2Drag = -1;
	}
	

    protected override void OnPaint(PaintEventArgs e)
    {
        base.OnPaint(e);
        
        // Show control points
        if (this.showControlPoints && this.b.ControlPoints.Count > 0)
        {
            Rectangle[] r = new Rectangle[this.b.ControlPoints.Count];
            for (int i=0; i<this.b.ControlPoints.Count; i++)
            {
                Point p = ((Point)this.b.ControlPoints[i]);
                r[i] = new Rectangle(p.X - 2, p.Y -2, 5, 5);
                e.Graphics.DrawString(i.ToString(), this.Font, Brushes.Black, (float) p.X + 7, (float) p.Y - 3);
            }
            e.Graphics.DrawRectangles(Pens.Black, r);
        }
        
        // Show lines between control points
        if (this.b.ControlPoints.Count > 1)
        {
            Point[] p = (Point[]) this.b.ControlPoints.ToArray(typeof(Point));
            e.Graphics.DrawLines(Pens.Black, p);
        }
        
        if (this.showCurvePoints)
        {
        
        }
        
        if (this.showConstructionPoints)
        {
			// Show BezierPoint
			if (!this.b.BezierPoint.IsEmpty)
			{
				Rectangle r = new Rectangle(this.b.BezierPoint.X - 2, this.b.BezierPoint.Y -2, 5, 5);
				e.Graphics.DrawRectangle(Pens.Red, r);
			}

			// Show ConstructionLines
			if (this.b.ConstructionPoints.Count > 0)
			{
				for (int i=0; i<this.b.ConstructionPoints.Count; i++)
				{
					ArrayList a = (ArrayList)this.b.ConstructionPoints[i];

					if (a.Count > 1)
					{
						Point[] p = (Point[]) (a.ToArray(typeof(Point)));
						e.Graphics.DrawLines(Pens.Gray, p);
					}
				}
			}
		}
		
		if (this.showCurvePoints)
		{
            if (this.b.CurvePoints.Count > 1)
            {
                Point[] p = (Point[]) (this.b.CurvePoints.ToArray(typeof(Point)));
                e.Graphics.DrawLines(Pens.Green, p);
            }
		}
    }
}